

(deffunction parse (?input)
             (bind ?exploded (explode$ ?input))
             (loop-for-count (?cnt 1 (length$ ?exploded)) do
                     (assert (wordAndPos (word (nth$ ?cnt ?exploded)) (pos ?cnt) ))
             )
)



(deffunction ask (?question)
             (printout t ?question crlf)
             (bind ?response (readline t))
             (parse ?response)
)

(deftemplate interm "intermediate assertions beschrijven"
             (slot name
                   (type SYMBOL)
             )
             (multislot keywords
                        (type SYMBOL)
             )
)

(deftemplate intermFact "tussenlaag feiten"
             (slot name ;zelfde naam als in interms
                   (type SYMBOL)
             )
             (slot value ;waarde van dit feit (bijvoorbeeld true of false of green)
                   (type SYMBOL)
             )
)



(defrule eliminate "verwijder alle oude (gelexte) woorden"
         (declare (salience 100))
         (or ?lex   <- (wordAndPos (word ?n) (pos ?p))
             ?lex   <- (lexAndPos (word ?n) (pos ?p))
         )
         =>
         (retract ?lex)
 )

(defrule couldNotParse "ik heb echt geen idee waar je het over hebt"
         (declare (salience 0))
         (not (lexAndPos (word ?n) (pos ?p)))
         (not (notParseable y))
         (stage 2)
         (not (problem solved))
         =>
         (ask "I did not understand what you just said. Could you try to give a more precise answer?")
)

(defrule parseYesAndNo "zorg ervoor dat positieve en negatieve woorden wanneer ze tegelijk voorkomen als negatief worden geinterpreteerd"
         (declare (salience 950))
         ?positive <- (lexAndPos (word yes) (pos ?p1))
         ?negative <- (lexAndPos (word no) (pos ?p2))
         =>
         (retract ?positive)
)

; Creatie van situatie


(defrule parseEmptyPaper
         (declare (salience 902))
         ?blank <- (lexAndPos (word blank))
         ?paper <- (lexAndPos (word paper))
         =>
         (retract ?blank)
         (retract ?paper)
         (assert (intermFact (name printerOn) (value yes)))
         (assert (intermFact (name blank)     (value yes)))
         (assert (intermFact (name paperOut)  (value yes)))
)

(defrule parseCannotPrint
         (declare (salience 900)) ; laagste salience want als er op iets anders gematched kan worden met die no, dan wil je dat (de printer is een algemene term)
         (not (intermFact (name cannotPrint)))
         (or      ?printing <- (lexAndPos (word printing) (pos ?p1))
                  ?printing <- (lexAndPos (word printer) (pos ?p1))
         )
         ?negative <- (lexAndPos (word no) (pos ?p2))
         =>
         (retract ?printing)
         (retract ?negative)
         (assert (intermFact (name cannotPrint) (value yes)))
)

(defrule parsePrinterOn
         (declare (salience 902))
         (not (intermFact (name printerOn)))
         ?printer <- (intermFact (name cannotPrint) (value yes))
         ?answer     <- (lexAndPos (word ?val) (pos ?p1))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name printerOn) (value ?val)))
)

(defrule parsePrinterNotOn1
         (declare (salience 903))
         ?printer <- (lexAndPos (word printer))
         ?no <- (lexAndPos (word no))
         ?off <- (lexAndPos (word on))
         =>
         (retract ?printer)
         (retract ?no)
         (retract ?off)
         (assert (intermFact (name printerOn) (value no)))
)

(defrule parsePrinterNotOn2
         (declare (salience 902))
         ?printer <- (lexAndPos (word printer))
         ?off <- (lexAndPos (word off))
         =>
         (retract ?printer)
         (retract ?off)
         (assert (intermFact (name printerOn) (value no)))
)

(defrule parseCanTurnPrinterOn
         (declare (salience 902))
         (not (intermFact (name turnedOn)))
         ?printer <- (intermFact (name printerOn) (value no))
         ?answer     <- (lexAndPos (word ?val) (pos ?p1))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name turnedOn) (value ?val)))
)

(defrule parsePrinterNotTurnOn
         (declare (salience 903))
         (not (intermFact (name turnedOn)))
         ?printer <- (lexAndPos (word printer))
         ?no      <- (lexAndPos (word no))
         ?turn    <- (lexAndPos (word turnOn))
         =>
         (retract ?printer)
         (retract ?no)
         (retract ?turn)
         (assert (intermFact (name printerOn) (value no)))
         (assert (intermFact (name turnedOn) (value no)))
)

(defrule parseCableOk
         (declare (salience 902))
         (not (intermFact (name cableOk)))
         ?turnedOn      <- (intermFact (name turnedOn) (value no))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name cableOk) (value ?val)))
)

(defrule parsePaperAv
         (declare (salience 902))
         (not (intermFact (name paperAv)))
         ?turnedOn      <- (intermFact (name printerOn) (value yes))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name paperAv) (value ?val)))
)

(defrule parsePaperout
         (declare (salience 903))
         (not (intermFact (name paperOut)))
         ?turnedOn      <- (intermFact (name printerOn) (value yes))
         ?paperAv      <- (intermFact (name paperAv)   (value yes))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name paperOut) (value ?val)))
)

(defrule parsePaperout1
         (declare (salience 902))
         (not (intermFact (name paperOut)))
         ?paper         <- (lexAndPos (word paper))
         ?answer        <- (lexAndPos (word no))
         =>
         (retract ?answer)
         (retract ?paper)
         (assert (intermFact (name printerOn) (value yes)))
         (assert (intermFact (name paperOut) (value no)))
)

(defrule parsePaperoutput
         (declare (salience 903))
         (not (intermFact (name blank)))
         ?turnedOn      <- (intermFact (name printerOn) (value yes))
         ?paperOut      <- (intermFact (name paperOut)   (value yes))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name blank) (value ?val)))
)

(defrule parseCartridge
         (declare (salience 904))
         (not (intermFact (name cartridge)))
         ?turnedOn      <- (intermFact (name printerOn)  (value yes))
         ?paperOut      <- (intermFact (name paperOut)   (value yes))
         ?blank         <- (intermFact (name blank)      (value yes))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name cartridge) (value ?val)))
)

(defrule parsePaperoutputgarbage
         (declare (salience 904))
         (not (intermFact (name seated)))
         ?turnedOn      <- (intermFact (name printerOn)  (value yes))
         ?paperOut      <- (intermFact (name paperOut)   (value yes))
         ?blank         <- (intermFact (name blank)      (value no))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name seated) (value ?val)))
)

(defrule parsePaperjam
         (declare (salience 904))
         (not (intermFact (name paperJam)))
         ?turnedOn      <- (intermFact (name printerOn) (value yes))
         ?paperAv      <- (intermFact (name paperAv)   (value yes))
         ?paperOut      <- (intermFact (name paperOut)   (value no))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name paperJam) (value ?val)))
)

(defrule parseNetwork
         (declare (salience 904))
         (not (intermFact (name isNetwork)))
         ?turnedOn      <- (intermFact (name printerOn) (value yes))
         ?paperAv      <- (intermFact (name paperAv)   (value yes))
         ?paperOut      <- (intermFact (name paperOut)   (value no))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name isNetwork) (value ?val)))
)

(defrule parseConnectedToPC
         (declare (salience 906))
         (not (intermFact (name isConnected)))
         ?turnedOn      <- (intermFact (name printerOn) (value yes))
         ?paperAv       <- (intermFact (name paperAv)   (value yes))
         ?paperOut      <- (intermFact (name paperOut)   (value no))
         ?isNetwork     <- (intermFact (name isNetwork)   (value no))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name isConnected) (value ?val)))
)

(defrule parseHasDrivers
         (declare (salience 906))
         (not (intermFact (name hasDrivers)))
         ?turnedOn      <- (intermFact (name printerOn) (value yes))
         (or (and ?paperAv   <- (intermFact (name paperAv)   (value yes))
                  ?paperOut  <- (intermFact (name paperOut)  (value no))
                  (or ?doDrivers <- (intermFact (name isConnected) (value yes))
                      ?doDrivers <- (intermFact (name isTimeout) (value no))
                  )
             )
             (and ?blank  <- (intermFact (name blank) (value no))
                  ?seated <- (intermFact (name seated) (value yes))
             )
         )
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name hasDrivers) (value ?val)))
)

(defrule parseTimeout
         (declare (salience 906))
         (not (intermFact (name isTimeout)))
         ?turnedOn      <- (intermFact (name printerOn) (value yes))
         ?paperAv       <- (intermFact (name paperAv)   (value yes))
         ?paperOut      <- (intermFact (name paperOut)   (value no))
         ?isNetwork     <- (intermFact (name isNetwork)   (value yes))
         ?answer        <- (lexAndPos (word ?val) (pos ?p2))
         (test (or (eq ?val yes) (eq ?val no)))
         =>
         (retract ?answer)
         (assert (intermFact (name isTimeout) (value ?val)))
)



; (deffacts interms
;          (interm (name printerBroken)  (keywords broken printer))
;          (interm (name cartridgeEmpty) (keywords cartridge empty))
;          (interm (name paperJam)       (keywords paper stuck))
;          (interm (name cannotPrint)    (keywords printing unable))
;          (interm (name printerOff)     (keywords printer off))
; )

